# PyTacToe
PyTacToe is a Python library for k-in-a-row family of games, including (m,n,k)-game (Gomoku, tic-tac-toe), Connect6, Pente, Ultimate tic-tac-toe and some others.

## Features
- Customization: most of the games are played on an n-by-m board including ∞-by-∞ with k-in-a-row to win with N players.
- Move verifying, getting legal moves, checking wins and draws
- Tracking board history and reverting moves
- Creating and parsing FEN
- Engine evaluation



## Installing
Install the latest release using pip:
```
pip install pytactoe
```

## Usage
### Basic features for (m,n,k)-game

Initializing a (4,3,3)-game
```python
>>> from pytactoe import TicTacToe
>>> game = TicTacToe(m=4, n=3, k=3, infinite=False, players=2)
```

Making and unmaking moves
```python
>>> game.move(0, 2)  # X moves to (0, 2)
>>> game.move(1, 1)  # O moves to (1, 1)
>>> game.move(0, 0)  # X moves to (0, 0)
>>> game.pop()       # Taking the last move back
```

Showing a simple ASCII board
```python
>>> game
..X.
.O..
....
```

Creating and parsing FEN
```python
>>> game.fen()
'2X1/1O2/4 X 2'

>>> TicTacToe.from_fen('2X1/1O2/4 X 2')
..X.
.O..
....
```

Getting legal moves
```python
>>> game.legalmoves()
[(0, 0), (0, 1), (0, 3), (1, 0), (1, 2), (1, 3), (2, 0), (2, 1), (2, 2), (2, 3)]
```

Getting the full game state
```python
>>> print(game.get_attributes())
infinite: False
m: 4
n: 3
board: [[., ., X, .], [., O, ., .], [., ., ., .]]
k: 3
turn: X
players: 2
ended: False
draw: False
winned: False
winner: .
move_count: 2
fullmove_number: 2
move_history: [(0, 2), (1, 1)]
```

Showing a fancy Unicode board
```python
>>> print(game.unicode_board())
🟫🟫⬜🟫
🟫⬛🟫🟫
🟫🟫🟫🟫
```

### Connect6
All other game classes are derived from the (m,n,k)-game, so pretty much everything works the same. But in Connect, there must be only two players.

Initializing a Connect6 game. 
```python
>>> from pytactoe import Connect
>>> c6 = Connect(m=19, n=19, k=6, infinite=False, p=2, q=1)
```

Regular moves in Connect are replaced with sequences
```python
>>> c6.move_sequence([(0, 3)])
>>> c6.move_sequence([(1, 2), (2, 3)])
>>> c6.move_sequence([(0, 4), (0, 5)])
>>> c6.move_sequence([(5, 5), (6, 6)])
>>> c6.pop()
```

### Pente

Pente works exactly the same as (m,n,k)-game, except there is custodial capture mechanic and wins by the number of captures:
```python
>>> from pytactoe import Pente
>>> pente = Pente.from_fen('XOO1/4/4/4 X 2')
>>> pente.move(0,3)
>>> pente
X..X
....
....
....
```

### Other games
Some other variants:

```python
>>> from pytactoe import UltimateTicTacToe, MisereTicTacToe, ImpartialTicTacToe, WildTicTacToe
>>> ult = UltimateTicTacToe(m=3, n=3, k=3, which_local_board=(0,0), players=2) # cannot be infinite
>>> mis = MisereTicTacToe(m=3, n=3, k=3, infinite = False, players=2)
>>> wild = WildTicTacToe(m=3, n=3, k=3, infinite = False, players=2)
>>> imp = ImpartialTicTacToe(m=3, n=3, k=3, infinite = False, players=2)
```

## License

PyTacToe is available under the MIT license. For the full text, check out `LICENSE`.